#include <amxmodx>
#include <amxmisc>
#include <cellarray>

#include <tutor_queue>

#define PLUGIN "Tutor Queue"
#define VERSION "1.0"
#define AUTHOR "R3X"


#define MAXLEN_TUTOR 512
#define TASK_TUTOR 467666646

new Array:gQueueData[33];
new Array:gQueueString[33];
new bool:gbQueueInit = false;


enum {
	_FIELD_PRIORITY,
	_FIELD_COLOR,
	_FIELD_TIME
}

new gszMessage[MAXLEN_TUTOR];

new Float:LastTime 		= 1.5;
new TutorPriority:LastPriority 	= TPRIORITY_NORMAL;
new TutorColor:LastColor	= TUTOR_YELLOW;


public plugin_init() {
	register_plugin(PLUGIN, VERSION, AUTHOR);
}

public plugin_precache(){	
	precache_generic("gfx/career/icon_!.tga");
	precache_generic("gfx/career/icon_!-bigger.tga");
	precache_generic("gfx/career/icon_i.tga");
	precache_generic("gfx/career/icon_i-bigger.tga");
	precache_generic("gfx/career/icon_skulls.tga");
	
	precache_generic("gfx/career/round_corner_ne.tga");
	precache_generic("gfx/career/round_corner_nw.tga");
	precache_generic("gfx/career/round_corner_se.tga");
	precache_generic("gfx/career/round_corner_sw.tga");
	
	precache_generic("resource/TutorScheme.res");
	precache_generic("resource/UI/TutorTextWindow.res");	
}

public plugin_natives(){
	register_native("set_tutor", "_set_tutor", 1);
	register_native("show_tutor", "_show_tutor");
	register_native("flush_tutor_queue", "_flush_tutor_queue", 1);
}


flushArrays(id){
	ArrayClear(gQueueData[id]);
	ArrayClear(gQueueString[id]);
}

public _flush_tutor_queue(id){
	static maxplayers = 0; 	
	if(!maxplayers) 
		maxplayers 	= get_maxplayers();
		
	tutorInit(maxplayers);
	
	if(1 <= id <= maxplayers)
		flushArrays(id)
	
}
public _set_tutor(TutorColor:color, Float:fTime, TutorPriority:priority){
	LastTime = fTime
	LastPriority = priority;
	LastColor = color;
}

public _show_tutor(plugin, params){
	if(params < 2)
		return 0;
	
	new id = get_param(1);
	
	vdformat(gszMessage, MAXLEN_TUTOR-1, 2, 3);
	tutorMake(id, LastColor, LastTime, gszMessage);
	return 1;
}


tutorMake(id, TutorColor:Color, Float:fTime = 0.0, const szText[]){	
	static maxplayers = 0; 	
	if(!maxplayers) 
		maxplayers 	= get_maxplayers();
	
	tutorInit(maxplayers);
	
	
	new iStart, iEnd;
	if(is_user_connected(id))
		iStart = id, iEnd = id;
	else
		iStart = 1, iEnd = maxplayers;
	
	new Data[3];
	Data[_FIELD_PRIORITY] = _:LastPriority;
	Data[_FIELD_COLOR] = _:Color;
	Data[_FIELD_TIME] = _:fTime;
	
	for(new i=iStart; i<=iEnd;i++)
		tutorAdd2Queue(i, Data, szText);
}

public taskTutorQueue(id){	
	id -= TASK_TUTOR;
	
	if(!is_user_connected(id)){
		flushArrays(id);
		return;
	}
	
	new iSize = ArraySize(gQueueData[id]);
	if(iSize == 0)
		return;
		
	ArrayDeleteItem(gQueueData[id], 0);
	ArrayDeleteItem(gQueueString[id], 0);
	
	tutorClose(id);
	
	if(iSize > 1)
		set_task(0.5, "taskTutorQueueDelayed", id+TASK_TUTOR);
}

public taskTutorQueueDelayed(id){
	id -= TASK_TUTOR;
	
	if(!is_user_connected(id)){
		ArrayClear(gQueueData[id]);
		ArrayClear(gQueueString[id]);
		return;
	}
	
	tutorStartTask(id);
}
tutorStartTask(id){
	new Data[3];
	ArrayGetArray(gQueueData[id], 0, Data);
	ArrayGetString(gQueueString[id], 0, gszMessage, MAXLEN_TUTOR-1);
	
	tutorShow(id, TutorColor:Data[_FIELD_COLOR], gszMessage);
	set_task(Float:Data[_FIELD_TIME], "taskTutorQueue", id+TASK_TUTOR);
}

tutorInit(maxplayers){		
	if(!gbQueueInit){
		for(new i = 1; i <= maxplayers; i++){
			gQueueData[i] = ArrayCreate(3); 
			gQueueString[i] = ArrayCreate(MAXLEN_TUTOR); 
			
		}
		gbQueueInit = true;
	}
}


tutorAdd2Queue(id, const Data[3], const szMessage[]){
	new place = tutorGetPlace(id, LastPriority);
	
	new bool:duringDisplay = !!task_exists(id+TASK_TUTOR);
	
	if(duringDisplay && TutorPriority:Data[_FIELD_PRIORITY] == TPRIORITY_SUPERCEDE){
		new iSize = ArraySize(gQueueData[id]);
		if(iSize){
			ArrayDeleteItem(gQueueData[id], 0);
			ArrayDeleteItem(gQueueString[id], 0);
			
			remove_task(id+TASK_TUTOR);
			duringDisplay = false;
		}
	}
	
	if(place == -1){
		ArrayPushArray(gQueueData[id], Data);
		ArrayPushString(gQueueString[id], szMessage);
	}else{
		ArrayInsertArrayBefore(gQueueData[id], place, Data);
		ArrayInsertStringBefore(gQueueString[id], place, szMessage);
	}
	
	if(!duringDisplay)
		tutorStartTask(id);
	
	return place;
}

//-1 PUSH, other INSERT BEFORE
tutorGetPlace(id, TutorPriority:priority){
	new iSize = ArraySize(gQueueData[id]);
	if(iSize == 0)
		return -1;
		
	if(priority == TPRIORITY_SUPERCEDE)
		return 0;
		
	new Data[3];
	for(new i=1;i<iSize;i++){
		ArrayGetArray(gQueueData[id], i, Data);
		
		if(Data[_FIELD_PRIORITY] < _:priority)
			return i;
	}
	
	return -1;
}







stock tutorShow(id, TutorColor:Color, const szMessage[]){
	static MsgTutor = 0; 	
	if(!MsgTutor) 	
		MsgTutor	= get_user_msgid("TutorText");
		
	message_begin(MSG_ONE_UNRELIABLE, MsgTutor,_,id);
	write_string(szMessage);
	write_byte(0);
	write_short(0);
	write_short(0);
	write_short(_:Color);
	message_end();
}

stock tutorClose(id){
	static MsgTutClose = 0; 	
	if(!MsgTutClose)
		MsgTutClose = get_user_msgid("TutorClose");
		
	message_begin(MSG_ONE_UNRELIABLE, MsgTutClose, _, id);
	message_end();
}
